﻿using System.Windows;
using System.Windows.Controls;
using CtrEditor.Convertidores;
using CtrEditor.Siemens;
using CtrEditor.Simulacion;


namespace CtrEditor.ObjetosSim
{
    /// <summary>
    /// Interaction logic for ucTransporteTTop.xaml
    /// </summary>
    /// 
    
    public class osTransporteTTop : osBase
    {
        private string _nombre = "Transporte TTOP";

        private float frictionCoefficient;
        private float velMax50hz;  // en metros por minuto      
        private float tiempoRampa;
        private bool esMarcha;

        private float _ancho;
        private float _alto;
        private float _left;
        private float _top;
        private float _angulo;
        private float _velocidadActual;

        private simRectangle Simulation_Transporte;


        public override float Left
        {
            get => _left;
            set
            {
                _left = value;
                CanvasSetLeftinMeter(value);
                OnPropertyChanged(nameof(Left));
            }
        }
        public override float Top
        {
            get => _top;
            set
            {
                _top = value;
                CanvasSetTopinMeter(value);
                OnPropertyChanged(nameof(Top));
            }
        }

        public float Ancho
        {
            get => _ancho;
            set
            {
                _ancho = value;
                OnPropertyChanged(nameof(Ancho));
            }
        }
        public float Alto
        {
            get => _alto;
            set
            {
                _alto = value;
                OnPropertyChanged(nameof(Alto));
            }
        }

        public float Angulo
        {
            get => _angulo;
            set
            {
                _angulo = value;
                OnPropertyChanged(nameof(Angulo));
            }
        }

        public float VelocidadActual
        {
            get => _velocidadActual;
            set {
                _velocidadActual = value;
                Simulation_Transporte?.SetSpeed(value);
                OnPropertyChanged(nameof(VelocidadActual));
            }
        }

        public override string Nombre
        {
            get => _nombre;
            set
            {
                if (_nombre != value)
                {
                    _nombre = value;
                    OnPropertyChanged(nameof(Nombre));
                }
            }
        }

        private void ActualizarGeometrias()
        {
            if (_visualRepresentation is ucTransporteTTop uc)
            {
                UpdateRectangle(Simulation_Transporte, uc.Transporte,Alto,Ancho,Angulo);
                Simulation_Transporte.Speed = VelocidadActual;
            }
        }

        public float FrictionCoefficient { get => frictionCoefficient; set => frictionCoefficient = value; }
        public float VelMax50hz { get => velMax50hz; set => velMax50hz = value; }
        public float TiempoRampa { get => tiempoRampa; set => tiempoRampa = value; }
        public bool EsMarcha { get => esMarcha; set => esMarcha = value; }

        public osTransporteTTop()
        {
            Ancho = 1;
            Alto = 0.10f;
        }

        public override void UpdateGeometryStart()
        {
            // Se llama antes de la simulacion
            ActualizarGeometrias();
        }
        public override void UpdateGeometryStep()
        {
        }
        public override void UpdatePLC(PLCModel plc) { }


        public override void UpdateControl()
        {
        }
        public override void ucLoaded()
        {
            // El UserControl ya se ha cargado y podemos obtener las coordenadas para 
            // crear el objeto de simulacion
            if (_visualRepresentation is ucTransporteTTop uc)
                Simulation_Transporte = AddRectangle(simulationManager, uc.Transporte, Alto, Ancho, Angulo);
        }

    }

    public partial class ucTransporteTTop : UserControl, IDataContainer
    {
        public osBase? Datos { get; set; }

        public ucTransporteTTop()
        {
            InitializeComponent();
            this.Loaded += OnLoaded;
        }
        private void OnLoaded(object sender, RoutedEventArgs e)
        {
            Datos?.ucLoaded();
        }
        public void Resize(float width, float height)
        {
            if (Datos is osTransporteTTop datos)
                datos.Ancho = PixelToMeter.Instance.calc.PixelsToMeters(width);
        }
        public void Move(float LeftPixels, float TopPixels)
        {
            if (Datos != null)
            {
                Datos.Left = PixelToMeter.Instance.calc.PixelsToMeters(LeftPixels);
                Datos.Top = PixelToMeter.Instance.calc.PixelsToMeters(TopPixels);
            }
        }
        public void Rotate(float Angle) { 
            if (Datos != null)
                if (Datos is osTransporteTTop datos)
                    datos.Angulo = Angle;
        }
        public void Highlight(bool State) { }
        public int ZIndex()
        {
            return 1;
        }

    }
}



